function [varargout] = LinearizedMSD(xbar,m,k1,k2,b1,dt)
%  [Alin,Blin,Clin,Dlin] = LinearizedMSD(xbar,m,k1,k2,b1,dt)
% OR
%  Glin = LinearizedMSD(xbar,m,k1,k2,b1,dt)
%
% This function computes a linearization for the nonlinear mass-spring-
% damper system. The system consists of M blocks connected by nonlinear 
% springs and dampers. In addition, each block is connected to a wall by 
% a nonlinear spring and damper. The system has an input force u and 
% an exogenous force F on the M^th block. 
%
% Inputs:
%  *xbar = 2M-by-1 vector specifying the equilibrium point for the
%       given value of rho, e.g. as returned byComputeMSDTrim.m.
%  *m = Mass of individual blocks, kg
%  *k1 = Linear spring constant, N/m
%  *k2 = Cubic spring constant, N/m^3
%  *b1 = Linear damping constant, N/(m/s)
%  *dt = Sample time for discrete-time linearization, sec.
%     Set dt=[] to obtain a continuous-time linearization.
%     [ Optional input. Default is dt = [] ]
%
% Output:
%  *Alin,Blin,Clin,Dlin = State matrices for the linearization from input 
%      force u on block M to output of (linearized) position of block M.  
%      The linearization is performed around the equilibrium xbar. The
%      linearization is in continuous-time if dt=[] and is in discrete-
%      time otherwise.
%  *Glin = Linearized dynamics returned as an state-space system, i.e.
%      Glin = ss(Alin,Blin,Clin,Dlin). 
%

% Set default input
if nargin==5
    dt = [];
end

% # of Masses
M = numel(xbar)/2;

% Equilbrium block positions
p = xbar(1:M);

% Stiffness matrix due to springs and damping matrix
Ak = zeros(M);
Ab = zeros(M);
for i=1:M 
    
    if i==1 && M==1
        % Special Case: Only a single block
        keff = klin(p(1),k1,k2);
        
        Ak(1,1) = -keff/m;
        Ab(1,1) = -b1/m;
    elseif i==1
        % Block 1
        keff1 = klin(p(1),k1,k2);
        keff12 = klin(p(2)-p(1),k1,k2);
        
        Ak(1,[1 2]) = [-keff1-keff12  keff12]/m;
        Ab(1,[1 2]) = [-2 1]*b1/m;
    elseif i==M
        % Block M
        keffM = klin(p(M),k1,k2);
        keffMm = klin(p(M)-p(M-1),k1,k2);
        
        Ak(M,[M-1 M]) = [keffMm -keffM-keffMm]/m;
        Ab(M,[M-1 M]) = [1 -2]*b1/m;
    else
        % Block i
        keffi = klin(p(i),k1,k2);
        keffim = klin(p(i)-p(i-1),k1,k2);
        keffip = klin(p(i+1)-p(i),k1,k2);
        
        Ak(i,i+[-1 0 1]) = [keffim -(keffi+keffim+keffip) keffip]/m;
        Ab(i,i+[-1 0 1]) = [1 -3 1]*b1/m;
    end    
end

% State matrix
Alin = [zeros(M) eye(M); Ak Ab];

% Input matrix: Force u on block M
Blin = zeros(2*M,1);
Blin(2*M,1) = 1/m;  

% Output matrix: Measurementof block M position 
Clin = zeros(1,2*M);
Clin(M) = 1; 

% Feedthrough matrix
Dlin = 0;

% Transfer Function
if isempty(dt)
    % Continuous-time linearization
    if nargout==1        
        Glin = ss(Alin,Blin,Clin,Dlin);
        varargout{1} = Glin;
    else
        varargout = {Alin,Blin,Clin,Dlin};
    end
else
    % Discrete-time linearization
    Alindt = eye(2*M) + dt*Alin;
    Blindt = dt*Blin;
    if nargout==1
        Glin = ss(Alindt,Blindt,Clin,Dlin,dt);    
        varargout{1} = Glin;
    else
        varargout = {Alindt,Blindt,Clin,Dlin};
    end        
end


%% Local function to compute linearized spring constant
function kout = klin(u,k1,k2)
kout = k1+3*k2*u^2;

